@@ -1,12 +1,18 @@
+import os
 from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.core.exceptions import ValidationError
+from django.db import IntegrityError
 from django.core.urlresolvers import reverse
 from django.test import TestCase
 
+from friendship.exceptions import AlreadyExistsError
 from friendship.models import Friend, Follow, FriendshipRequest
 
 
+TEST_TEMPLATES = os.path.join(os.path.dirname(__file__), 'templates')
+
+
 class login(object):
     def __init__(self, testcase, user, password):
         self.testcase = testcase
@@ -118,6 +124,12 @@ class FriendshipModelTests(BaseTestCase):
         self.assertEqual(Friend.objects.friends(self.user_amy), [])
         req3.reject()
 
+        # Duplicated requests raise a more specific subclass of IntegrityError.
+        with self.assertRaises(IntegrityError):
+            Friend.objects.add_friend(self.user_susan, self.user_amy)
+        with self.assertRaises(AlreadyExistsError):
+            Friend.objects.add_friend(self.user_susan, self.user_amy)
+
         self.assertFalse(Friend.objects.are_friends(self.user_susan, self.user_amy))
         self.assertEqual(len(Friend.objects.rejected_requests(self.user_amy)), 1)
         self.assertEqual(len(Friend.objects.rejected_requests(self.user_amy)), 1)
@@ -140,6 +152,35 @@ class FriendshipModelTests(BaseTestCase):
         with self.assertRaises(ValidationError):
             Friend.objects.create(to_user=self.user_bob, from_user=self.user_bob)
 
+    def test_multiple_friendship_requests(self):
+        """ Ensure multiple friendship requests are handled properly """
+        ### Bob wants to be friends with Steve
+        req1 = Friend.objects.add_friend(self.user_bob, self.user_steve)
+
+        # Ensure neither have friends already
+        self.assertEqual(Friend.objects.friends(self.user_bob), [])
+        self.assertEqual(Friend.objects.friends(self.user_steve), [])
+
+        # Ensure FriendshipRequest is created
+        self.assertEqual(FriendshipRequest.objects.filter(from_user=self.user_bob).count(), 1)
+        self.assertEqual(FriendshipRequest.objects.filter(to_user=self.user_steve).count(), 1)
+        self.assertEqual(Friend.objects.unread_request_count(self.user_steve), 1)
+
+        # Steve also wants to be friends with Bob before Bob replies
+        req2 = Friend.objects.add_friend(self.user_steve, self.user_bob)
+
+        # Ensure they aren't friends at this point
+        self.assertFalse(Friend.objects.are_friends(self.user_bob, self.user_steve))
+
+        # Accept the request
+        req1.accept()
+
+        # Ensure neither have pending requests
+        self.assertEqual(FriendshipRequest.objects.filter(from_user=self.user_bob).count(), 0)
+        self.assertEqual(FriendshipRequest.objects.filter(to_user=self.user_steve).count(), 0)
+        self.assertEqual(FriendshipRequest.objects.filter(from_user=self.user_steve).count(), 0)
+        self.assertEqual(FriendshipRequest.objects.filter(to_user=self.user_bob).count(), 0)
+
     def test_following(self):
         # Bob follows Steve
         req1 = Follow.objects.add_follower(self.user_bob, self.user_steve)
@@ -151,6 +192,12 @@ class FriendshipModelTests(BaseTestCase):
         self.assertTrue(Follow.objects.follows(self.user_bob, self.user_steve))
         self.assertFalse(Follow.objects.follows(self.user_steve, self.user_bob))
 
+        # Duplicated requests raise a more specific subclass of IntegrityError.
+        with self.assertRaises(IntegrityError):
+            Follow.objects.add_follower(self.user_bob, self.user_steve)
+        with self.assertRaises(AlreadyExistsError):
+            Follow.objects.add_follower(self.user_bob, self.user_steve)
+
         # Remove the relationship
         self.assertTrue(Follow.objects.remove_follower(self.user_bob, self.user_steve))
         self.assertEqual(len(Follow.objects.followers(self.user_steve)), 0)
@@ -178,12 +225,23 @@ class FriendshipViewTests(BaseTestCase):
         response = self.client.get(url)
         self.assertResponse200(response)
 
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_LIST_NAME='object_list', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object_list' in response.context)
+
     def test_friendship_view_friends(self):
         url = reverse('friendship_view_friends', kwargs={'username': self.user_bob.username})
 
         # test that the view requires authentication to access it
         response = self.client.get(url)
         self.assertResponse200(response)
+        self.assertTrue('user' in response.context)
+
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_NAME='object', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object' in response.context)
 
     def test_friendship_add_friend(self):
         url = reverse('friendship_add_friend', kwargs={'to_username': self.user_amy.username})
@@ -205,6 +263,25 @@ class FriendshipViewTests(BaseTestCase):
             redirect_url = reverse('friendship_request_list')
             self.assertTrue(redirect_url in response['Location'])
 
+    def test_friendship_add_friend_dupe(self):
+        url = reverse('friendship_add_friend', kwargs={'to_username': self.user_amy.username})
+
+        with self.login(self.user_bob.username, self.user_pw):
+            # if we don't POST the view should return the
+            # friendship_add_friend view
+
+            # on POST accept the friendship request and redirect to the
+            # friendship_request_list view
+            response = self.client.post(url)
+            self.assertResponse302(response)
+            redirect_url = reverse('friendship_request_list')
+            self.assertTrue(redirect_url in response['Location'])
+
+            response = self.client.post(url)
+            self.assertResponse200(response)
+            self.assertTrue('errors' in response.context)
+            self.assertEqual(response.context['errors'], ['Friendship already requested'])
+
     def test_friendship_requests(self):
         url = reverse('friendship_request_list')
 
@@ -311,6 +388,11 @@ class FriendshipViewTests(BaseTestCase):
         response = self.client.get(url)
         self.assertResponse200(response)
 
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_NAME='object', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object' in response.context)
+
     def test_friendship_following(self):
         url = reverse('friendship_following', kwargs={'username': 'bob'})
 
@@ -318,6 +400,11 @@ class FriendshipViewTests(BaseTestCase):
         response = self.client.get(url)
         self.assertResponse200(response)
 
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_NAME='object', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object' in response.context)
+
     def test_follower_add(self):
         url = reverse('follower_add', kwargs={'followee_username': self.user_amy.username})
 
@@ -336,6 +423,11 @@ class FriendshipViewTests(BaseTestCase):
             redirect_url = reverse('friendship_following', kwargs={'username': self.user_bob.username})
             self.assertTrue(redirect_url in response['Location'])
 
+            response = self.client.post(url)
+            self.assertResponse200(response)
+            self.assertTrue('errors' in response.context)
+            self.assertEqual(response.context['errors'], ["User 'bob' already follows 'amy'"])
+
     def test_follower_remove(self):
         # create a follow relationship so we can test removing a follower
         follow = Follow.objects.add_follower(self.user_bob, self.user_amy)
