@@ -1,33 +1,45 @@
 from django.contrib.auth.decorators import login_required
+from django.conf import settings
 try:
     from django.contrib.auth import get_user_model
     user_model = get_user_model()
 except ImportError:
     from django.contrib.auth.models import User
     user_model = User
+
 from django.shortcuts import render, get_object_or_404, redirect
 
+from friendship.exceptions import AlreadyExistsError
 from friendship.models import Friend, Follow, FriendshipRequest
 
+get_friendship_context_object_name = lambda: getattr(settings, 'FRIENDSHIP_CONTEXT_OBJECT_NAME', 'user')
+get_friendship_context_object_list_name = lambda: getattr(settings, 'FRIENDSHIP_CONTEXT_OBJECT_LIST_NAME', 'users')
+
 
 def view_friends(request, username, template_name='friendship/friend/user_list.html'):
     """ View the friends of a user """
     user = get_object_or_404(user_model, username=username)
     friends = Friend.objects.friends(user)
+    return render(request, template_name, {get_friendship_context_object_name(): user, 'friends': friends})
 
-    return render(request, template_name, {'user': user, 'friends': friends})
 
 
 @login_required
 def friendship_add_friend(request, to_username, template_name='friendship/friend/add.html'):
     """ Create a FriendshipRequest """
+    ctx = {'to_username': to_username}
+
     if request.method == 'POST':
         to_user = user_model.objects.get(username=to_username)
         from_user = request.user
-        Friend.objects.add_friend(from_user, to_user)
-        return redirect('friendship_request_list')
+        try:
+            Friend.objects.add_friend(from_user, to_user)
+        except AlreadyExistsError as e:
+            ctx['errors'] = ["%s" % e]
+        else:
+            return redirect('friendship_request_list')
 
-    return render(request, template_name, {'to_username': to_username})
+    return render(request, template_name, ctx)
 
 
 @login_required
@@ -94,7 +106,7 @@ def followers(request, username, template_name='friendship/follow/followers_list
     user = get_object_or_404(user_model, username=username)
     followers = Follow.objects.followers(user)
 
-    return render(request, template_name, {'user': user, 'followers': followers})
+    return render(request, template_name, {get_friendship_context_object_name(): user, 'followers': followers})
 
 
 def following(request, username, template_name='friendship/follow/following_list.html'):
@@ -102,19 +114,25 @@ def following(request, username, template_name='friendship/follow/following_list
     user = get_object_or_404(user_model, username=username)
     following = Follow.objects.following(user)
 
-    return render(request, template_name, {'user': user, 'following': following})
+    return render(request, template_name, {get_friendship_context_object_name(): user, 'following': following})
 
 
 @login_required
 def follower_add(request, followee_username, template_name='friendship/follow/add.html'):
     """ Create a following relationship """
+    ctx = {'followee_username': followee_username}
+
     if request.method == 'POST':
         followee = user_model.objects.get(username=followee_username)
         follower = request.user
-        Follow.objects.add_follower(follower, followee)
-        return redirect('friendship_following', username=follower.username)
+        try:
+            Follow.objects.add_follower(follower, followee)
+        except AlreadyExistsError as e:
+            ctx['errors'] = ["%s" % e]
+        else:
+            return redirect('friendship_following', username=follower.username)
 
-    return render(request, template_name, {'followee_username': followee_username})
+    return render(request, template_name, ctx)
 
 
 @login_required
@@ -132,4 +150,4 @@ def follower_remove(request, followee_username, template_name='friendship/follow
 def all_users(request, template_name="friendship/user_actions.html"):
     users = user_model.objects.all()
 
-    return render(request, template_name, {'users': users})
+    return render(request, template_name, {get_friendship_context_object_list_name(): users})
